# SPDX-FileCopyrightText: Copyright © 2024 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import click
from clapper.click import verbosity_option
from clapper.logging import setup

logger = setup(__name__.split(".")[0], format="%(levelname)s: %(message)s")


@click.command(
    epilog="""Examples:

1. Provide information about the current installation:

   .. code:: sh

      mednet info

""",
)
@verbosity_option(logger=logger, expose_value=False)
def info(
    **_,
) -> None:  # numpydoc ignore=PR01
    """Provide information about the current installation."""

    import typing

    from ..utils.rc import load_rc
    from .database import _get_raw_databases
    from .utils import execution_metadata

    def _echo(left: str, right: str, color: str = "white") -> None:
        s = [
            click.style(left, bold=True),
            click.style(": ", bold=True),
            click.style(right, fg=color),
        ]
        click.echo("".join(s))

    m = execution_metadata()
    c = load_rc()

    _echo("mednet version", typing.cast(str, m["package-version"]), "green")
    _echo("platform", typing.cast(str, m["platform"]), "yellow")
    _echo(
        "accelerators",
        ", ".join(typing.cast(list[str], m["accelerators"])),
        "cyan",
    )

    if not c.path.exists():
        _echo("configuration", f"{str(c.path)} [MISSING]", "white")
    else:
        _echo("configuration", str(c.path), "white")
    dbs = _get_raw_databases()
    click.secho("configured databases:", bold=True)
    for k, v in dbs.items():
        if "datadir" not in v:
            # this database does not have a "datadir"
            continue

        if v["datadir"] is not None:
            _echo(f"  - {k} ({v['module']})", f"{v['datadir']}", "green")
        else:
            _echo(f"  - {k} ({v['module']})", "NOT installed", "red")

    click.secho("dependencies:", bold=True)
    python = typing.cast(dict[str, str], m["python"])
    _echo(
        "  - python",
        f"{python['version']} ({python['path']})",
        "white",
    )
    deps = typing.cast(dict[str, str], m["dependencies"])
    for name, version in deps.items():
        _echo(f"  - {name}", version, "white")
